FFmpeg
checkasm.c
Go to the documentation of this file.
1 /*
2  * Assembly testing and benchmarking tool
3  * Copyright (c) 2015 Henrik Gramner
4  * Copyright (c) 2008 Loren Merritt
5  *
6  * This file is part of FFmpeg.
7  *
8  * FFmpeg is free software; you can redistribute it and/or modify
9  * it under the terms of the GNU General Public License as published by
10  * the Free Software Foundation; either version 2 of the License, or
11  * (at your option) any later version.
12  *
13  * FFmpeg is distributed in the hope that it will be useful,
14  * but WITHOUT ANY WARRANTY; without even the implied warranty of
15  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
16  * GNU General Public License for more details.
17  *
18  * You should have received a copy of the GNU General Public License along
19  * with FFmpeg; if not, write to the Free Software Foundation, Inc.,
20  * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
21  *
22  * Copyright © 2018, VideoLAN and dav1d authors
23  * Copyright © 2018, Two Orioles, LLC
24  * All rights reserved.
25  *
26  * Redistribution and use in source and binary forms, with or without
27  * modification, are permitted provided that the following conditions are met:
28  *
29  * 1. Redistributions of source code must retain the above copyright notice, this
30  * list of conditions and the following disclaimer.
31  *
32  * 2. Redistributions in binary form must reproduce the above copyright notice,
33  * this list of conditions and the following disclaimer in the documentation
34  * and/or other materials provided with the distribution.
35  *
36  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
37  * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
38  * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
39  * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
40  * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
41  * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
42  * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
43  * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
44  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
45  * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
46  */
47 
48 #include "config.h"
49 #include "config_components.h"
50 
51 #ifndef _GNU_SOURCE
52 # define _GNU_SOURCE // for syscall (performance monitoring API), strsignal()
53 #endif
54 
55 #include <signal.h>
56 #include <stdarg.h>
57 #include <stdio.h>
58 #include <stdlib.h>
59 #include <string.h>
60 #include "checkasm.h"
61 #include "libavutil/avassert.h"
62 #include "libavutil/common.h"
63 #include "libavutil/cpu.h"
64 #include "libavutil/intfloat.h"
65 #include "libavutil/random_seed.h"
66 
67 #if HAVE_IO_H
68 #include <io.h>
69 #endif
70 #if HAVE_PRCTL
71 #include <sys/prctl.h>
72 #endif
73 
74 #if defined(_WIN32) && !defined(SIGBUS)
75 /* non-standard, use the same value as mingw-w64 */
76 #define SIGBUS 10
77 #endif
78 
79 #if HAVE_SETCONSOLETEXTATTRIBUTE && HAVE_GETSTDHANDLE
80 #include <windows.h>
81 #define COLOR_RED FOREGROUND_RED
82 #define COLOR_GREEN FOREGROUND_GREEN
83 #define COLOR_YELLOW (FOREGROUND_RED|FOREGROUND_GREEN)
84 #else
85 #define COLOR_RED 1
86 #define COLOR_GREEN 2
87 #define COLOR_YELLOW 3
88 #endif
89 
90 #if HAVE_UNISTD_H
91 #include <unistd.h>
92 #endif
93 
94 #if !HAVE_ISATTY
95 #define isatty(fd) 1
96 #endif
97 
98 #if ARCH_AARCH64
99 #include "libavutil/aarch64/cpu.h"
100 #elif ARCH_RISCV
101 #include "libavutil/riscv/cpu.h"
102 #endif
103 
104 #if ARCH_ARM && HAVE_ARMV5TE_EXTERNAL
105 #include "libavutil/arm/cpu.h"
106 
107 void (*checkasm_checked_call)(void *func, int dummy, ...) = checkasm_checked_call_novfp;
108 #endif
109 
110 /* Trade-off between speed and accuracy */
111 uint64_t bench_runs = 1U << 10;
112 
113 /* List of tests to invoke */
114 static const struct {
115  const char *name;
116  void (*func)(void);
117 } tests[] = {
118  /* NOTE: When adding a new test to this list here, it also needs to be
119  * added in tests/fate/checkasm.mak, otherwise it doesn't get executed
120  * as part of "make fate" or "make fate-checkasm". */
121 #if CONFIG_AVCODEC
122  #if CONFIG_AAC_DECODER
123  { "aacpsdsp", checkasm_check_aacpsdsp },
124  { "sbrdsp", checkasm_check_sbrdsp },
125  #endif
126  #if CONFIG_AAC_ENCODER
127  { "aacencdsp", checkasm_check_aacencdsp },
128  #endif
129  #if CONFIG_AC3DSP
130  { "ac3dsp", checkasm_check_ac3dsp },
131  #endif
132  #if CONFIG_ALAC_DECODER
133  { "alacdsp", checkasm_check_alacdsp },
134  #endif
135  #if CONFIG_APV_DECODER
136  { "apv_dsp", checkasm_check_apv_dsp },
137  #endif
138  #if CONFIG_AUDIODSP
139  { "audiodsp", checkasm_check_audiodsp },
140  #endif
141  #if CONFIG_BLOCKDSP
142  { "blockdsp", checkasm_check_blockdsp },
143  #endif
144  #if CONFIG_BSWAPDSP
145  { "bswapdsp", checkasm_check_bswapdsp },
146  #endif
147  #if CONFIG_CAVS_DECODER
148  { "cavsdsp", checkasm_check_cavsdsp },
149  #endif
150  #if CONFIG_DCA_DECODER
151  { "dcadsp", checkasm_check_dcadsp },
152  { "synth_filter", checkasm_check_synth_filter },
153  #endif
154  #if CONFIG_DIRAC_DECODER
155  { "diracdsp", checkasm_check_diracdsp },
156  #endif
157  #if CONFIG_EXR_DECODER
158  { "exrdsp", checkasm_check_exrdsp },
159  #endif
160  #if CONFIG_FDCTDSP
161  { "fdctdsp", checkasm_check_fdctdsp },
162  #endif
163  #if CONFIG_FLAC_DECODER
164  { "flacdsp", checkasm_check_flacdsp },
165  #endif
166  #if CONFIG_FMTCONVERT
167  { "fmtconvert", checkasm_check_fmtconvert },
168  #endif
169  #if CONFIG_G722DSP
170  { "g722dsp", checkasm_check_g722dsp },
171  #endif
172  #if CONFIG_H263DSP
173  { "h263dsp", checkasm_check_h263dsp },
174  #endif
175  #if CONFIG_H264CHROMA
176  { "h264chroma", checkasm_check_h264chroma },
177  #endif
178  #if CONFIG_H264DSP
179  { "h264dsp", checkasm_check_h264dsp },
180  #endif
181  #if CONFIG_H264PRED
182  { "h264pred", checkasm_check_h264pred },
183  #endif
184  #if CONFIG_H264QPEL
185  { "h264qpel", checkasm_check_h264qpel },
186  #endif
187  #if CONFIG_HEVC_DECODER
188  { "hevc_add_res", checkasm_check_hevc_add_res },
189  { "hevc_deblock", checkasm_check_hevc_deblock },
190  { "hevc_dequant", checkasm_check_hevc_dequant },
191  { "hevc_idct", checkasm_check_hevc_idct },
192  { "hevc_pel", checkasm_check_hevc_pel },
193  { "hevc_sao", checkasm_check_hevc_sao },
194  #endif
195  #if CONFIG_HPELDSP
196  { "hpeldsp", checkasm_check_hpeldsp },
197  #endif
198  #if CONFIG_HUFFYUVDSP
199  { "huffyuvdsp", checkasm_check_huffyuvdsp },
200  #endif
201  #if CONFIG_HUFFYUVENCDSP
202  { "huffyuvencdsp", checkasm_check_huffyuvencdsp },
203  #endif
204  #if CONFIG_IDCTDSP
205  { "idctdsp", checkasm_check_idctdsp },
206  #endif
207  #if CONFIG_JPEG2000_DECODER
208  { "jpeg2000dsp", checkasm_check_jpeg2000dsp },
209  #endif
210  #if CONFIG_LLAUDDSP
211  { "llauddsp", checkasm_check_llauddsp },
212  #endif
213  #if CONFIG_HUFFYUVDSP
214  { "llviddsp", checkasm_check_llviddsp },
215  #endif
216  #if CONFIG_LLVIDENCDSP
217  { "llvidencdsp", checkasm_check_llvidencdsp },
218  #endif
219  #if CONFIG_LPC
220  { "lpc", checkasm_check_lpc },
221  #endif
222  #if CONFIG_ME_CMP
223  { "motion", checkasm_check_motion },
224  #endif
225  #if CONFIG_MPEGVIDEO
226  { "mpegvideo_unquantize", checkasm_check_mpegvideo_unquantize },
227  #endif
228  #if CONFIG_MPEGVIDEOENCDSP
229  { "mpegvideoencdsp", checkasm_check_mpegvideoencdsp },
230  #endif
231  #if CONFIG_OPUS_DECODER
232  { "opusdsp", checkasm_check_opusdsp },
233  #endif
234  #if CONFIG_PIXBLOCKDSP
235  { "pixblockdsp", checkasm_check_pixblockdsp },
236  #endif
237  #if CONFIG_PNG_DECODER
238  { "png", checkasm_check_png },
239  #endif
240  #if CONFIG_QPELDSP
241  { "qpeldsp", checkasm_check_qpeldsp },
242  #endif
243  #if CONFIG_RV34DSP
244  { "rv34dsp", checkasm_check_rv34dsp },
245  #endif
246  #if CONFIG_RV40_DECODER
247  { "rv40dsp", checkasm_check_rv40dsp },
248  #endif
249  #if CONFIG_SVQ1_ENCODER
250  { "svq1enc", checkasm_check_svq1enc },
251  #endif
252  #if CONFIG_TAK_DECODER
253  { "takdsp", checkasm_check_takdsp },
254  #endif
255  #if CONFIG_UTVIDEO_DECODER
256  { "utvideodsp", checkasm_check_utvideodsp },
257  #endif
258  #if CONFIG_V210_DECODER
259  { "v210dec", checkasm_check_v210dec },
260  #endif
261  #if CONFIG_V210_ENCODER
262  { "v210enc", checkasm_check_v210enc },
263  #endif
264  #if CONFIG_VC1DSP
265  { "vc1dsp", checkasm_check_vc1dsp },
266  #endif
267  #if CONFIG_VP3DSP
268  { "vp3dsp", checkasm_check_vp3dsp },
269  #endif
270  #if CONFIG_VP6_DECODER
271  { "vp6dsp", checkasm_check_vp6dsp },
272  #endif
273  #if CONFIG_VP8DSP
274  { "vp8dsp", checkasm_check_vp8dsp },
275  #endif
276  #if CONFIG_VP9_DECODER
277  { "vp9dsp", checkasm_check_vp9dsp }, // all of the below
278  { "vp9_ipred", checkasm_check_vp9_ipred },
279  { "vp9_itxfm", checkasm_check_vp9_itxfm },
280  { "vp9_loopfilter", checkasm_check_vp9_loopfilter },
281  { "vp9_mc", checkasm_check_vp9_mc },
282  #endif
283  #if CONFIG_VIDEODSP
284  { "videodsp", checkasm_check_videodsp },
285  #endif
286  #if CONFIG_VORBIS_DECODER
287  { "vorbisdsp", checkasm_check_vorbisdsp },
288  #endif
289  #if CONFIG_VVC_DECODER
290  { "vvc_alf", checkasm_check_vvc_alf },
291  { "vvc_mc", checkasm_check_vvc_mc },
292  { "vvc_sao", checkasm_check_vvc_sao },
293  #endif
294 #endif
295 #if CONFIG_AVFILTER
296  #if CONFIG_SCENE_SAD
297  { "scene_sad", checkasm_check_scene_sad },
298  #endif
299  #if CONFIG_AFIR_FILTER
300  { "af_afir", checkasm_check_afir },
301  #endif
302  #if CONFIG_BLACKDETECT_FILTER
303  { "vf_blackdetect", checkasm_check_blackdetect },
304  #endif
305  #if CONFIG_BLEND_FILTER
306  { "vf_blend", checkasm_check_blend },
307  #endif
308  #if CONFIG_BWDIF_FILTER
309  { "vf_bwdif", checkasm_check_vf_bwdif },
310  #endif
311  #if CONFIG_COLORDETECT_FILTER
312  { "vf_colordetect", checkasm_check_colordetect },
313  #endif
314  #if CONFIG_COLORSPACE_FILTER
315  { "vf_colorspace", checkasm_check_colorspace },
316  #endif
317  #if CONFIG_EQ_FILTER
318  { "vf_eq", checkasm_check_vf_eq },
319  #endif
320  #if CONFIG_FSPP_FILTER
321  { "vf_fspp", checkasm_check_vf_fspp },
322  #endif
323  #if CONFIG_GBLUR_FILTER
324  { "vf_gblur", checkasm_check_vf_gblur },
325  #endif
326  #if CONFIG_HFLIP_FILTER
327  { "vf_hflip", checkasm_check_vf_hflip },
328  #endif
329  #if CONFIG_IDET_FILTER
330  { "vf_idet", checkasm_check_idet },
331  #endif
332  #if CONFIG_NLMEANS_FILTER
333  { "vf_nlmeans", checkasm_check_nlmeans },
334  #endif
335  #if CONFIG_THRESHOLD_FILTER
336  { "vf_threshold", checkasm_check_vf_threshold },
337  #endif
338  #if CONFIG_SOBEL_FILTER
339  { "vf_sobel", checkasm_check_vf_sobel },
340  #endif
341 #endif
342 #if CONFIG_SWSCALE
343  { "sw_gbrp", checkasm_check_sw_gbrp },
344  { "sw_range_convert", checkasm_check_sw_range_convert },
345  { "sw_rgb", checkasm_check_sw_rgb },
346  { "sw_scale", checkasm_check_sw_scale },
347  { "sw_xyz2rgb", checkasm_check_sw_xyz2rgb },
348  { "sw_yuv2rgb", checkasm_check_sw_yuv2rgb },
349  { "sw_yuv2yuv", checkasm_check_sw_yuv2yuv },
350  { "sw_ops", checkasm_check_sw_ops },
351 #endif
352 #if CONFIG_AVUTIL
353  { "aes", checkasm_check_aes },
354  { "crc", checkasm_check_crc },
355  { "fixed_dsp", checkasm_check_fixed_dsp },
356  { "float_dsp", checkasm_check_float_dsp },
357  { "lls", checkasm_check_lls },
358  { "av_tx", checkasm_check_av_tx },
359 #endif
360  { NULL }
361  /* NOTE: When adding a new test to this list here, it also needs to be
362  * added in tests/fate/checkasm.mak, otherwise it doesn't get executed
363  * as part of "make fate" or "make fate-checkasm". */
364 };
365 
366 /* List of cpu flags to check */
367 static const struct {
368  const char *name;
369  const char *suffix;
370  int flag;
371 } cpus[] = {
372 #if ARCH_AARCH64
373  { "ARMV8", "armv8", AV_CPU_FLAG_ARMV8 },
374  { "NEON", "neon", AV_CPU_FLAG_NEON },
375  { "DOTPROD", "dotprod", AV_CPU_FLAG_DOTPROD },
376  { "I8MM", "i8mm", AV_CPU_FLAG_I8MM },
377  { "SVE", "sve", AV_CPU_FLAG_SVE },
378  { "SVE2", "sve2", AV_CPU_FLAG_SVE2 },
379  { "SME", "sme", AV_CPU_FLAG_SME },
380  { "SME-I16I64", "sme_i16i64", AV_CPU_FLAG_SME_I16I64 },
381  { "CRC", "crc", AV_CPU_FLAG_ARM_CRC },
382  { "SME2", "sme2", AV_CPU_FLAG_SME2 },
383 #elif ARCH_ARM
384  { "ARMV5TE", "armv5te", AV_CPU_FLAG_ARMV5TE },
385  { "ARMV6", "armv6", AV_CPU_FLAG_ARMV6 },
386  { "ARMV6T2", "armv6t2", AV_CPU_FLAG_ARMV6T2 },
387  { "VFP", "vfp", AV_CPU_FLAG_VFP },
388  { "VFP_VM", "vfp_vm", AV_CPU_FLAG_VFP_VM },
389  { "VFPV3", "vfp3", AV_CPU_FLAG_VFPV3 },
390  { "NEON", "neon", AV_CPU_FLAG_NEON },
391 #elif ARCH_PPC
392  { "ALTIVEC", "altivec", AV_CPU_FLAG_ALTIVEC },
393  { "VSX", "vsx", AV_CPU_FLAG_VSX },
394  { "POWER8", "power8", AV_CPU_FLAG_POWER8 },
395 #elif ARCH_RISCV
396  { "RVI", "rvi", AV_CPU_FLAG_RVI },
397  { "misaligned", "misaligned", AV_CPU_FLAG_RV_MISALIGNED },
398  { "RV_zbb", "rvb_b", AV_CPU_FLAG_RVB_BASIC },
399  { "RVB", "rvb", AV_CPU_FLAG_RVB },
400  { "RV_zve32x","rvv_i32", AV_CPU_FLAG_RVV_I32 },
401  { "RV_zve32f","rvv_f32", AV_CPU_FLAG_RVV_F32 },
402  { "RV_zve64x","rvv_i64", AV_CPU_FLAG_RVV_I64 },
403  { "RV_zve64d","rvv_f64", AV_CPU_FLAG_RVV_F64 },
404  { "RV_zvbb", "rv_zvbb", AV_CPU_FLAG_RV_ZVBB },
405 #elif ARCH_MIPS
406  { "MMI", "mmi", AV_CPU_FLAG_MMI },
407  { "MSA", "msa", AV_CPU_FLAG_MSA },
408 #elif ARCH_X86
409  { "MMX", "mmx", AV_CPU_FLAG_MMX|AV_CPU_FLAG_CMOV },
410  { "MMXEXT", "mmxext", AV_CPU_FLAG_MMXEXT },
411  { "SSE", "sse", AV_CPU_FLAG_SSE },
412  { "SSE2", "sse2", AV_CPU_FLAG_SSE2|AV_CPU_FLAG_SSE2SLOW },
413  { "SSE3", "sse3", AV_CPU_FLAG_SSE3|AV_CPU_FLAG_SSE3SLOW },
414  { "SSSE3", "ssse3", AV_CPU_FLAG_SSSE3|AV_CPU_FLAG_ATOM },
415  { "SSE4.1", "sse4", AV_CPU_FLAG_SSE4 },
416  { "SSE4.2", "sse42", AV_CPU_FLAG_SSE42 },
417  { "AES-NI", "aesni", AV_CPU_FLAG_AESNI },
418  { "CLMUL", "clmul", AV_CPU_FLAG_CLMUL },
419  { "AVX", "avx", AV_CPU_FLAG_AVX },
420  { "XOP", "xop", AV_CPU_FLAG_XOP },
421  { "FMA3", "fma3", AV_CPU_FLAG_FMA3 },
422  { "FMA4", "fma4", AV_CPU_FLAG_FMA4 },
423  { "AVX2", "avx2", AV_CPU_FLAG_AVX2 },
424  { "AVX-512", "avx512", AV_CPU_FLAG_AVX512 },
425  { "AVX-512ICL", "avx512icl", AV_CPU_FLAG_AVX512ICL },
426 #elif ARCH_LOONGARCH
427  { "LSX", "lsx", AV_CPU_FLAG_LSX },
428  { "LASX", "lasx", AV_CPU_FLAG_LASX },
429 #elif ARCH_WASM
430  { "SIMD128", "simd128", AV_CPU_FLAG_SIMD128 },
431 #endif
432  { NULL }
433 };
434 
435 typedef struct CheckasmFuncVersion {
437  void *func;
438  int ok;
439  int cpu;
442 
443 /* Binary search tree node */
444 typedef struct CheckasmFunc {
445  struct CheckasmFunc *child[2];
447  uint8_t color; /* 0 = red, 1 = black */
448  char name[1];
449 } CheckasmFunc;
450 
451 /* Internal state */
452 static struct {
456  const char *current_test_name;
457  const char *bench_pattern;
461 
462  /* perf */
463  int nop_time;
464  int sysfd;
465 
466  int cpu_flag;
467  const char *cpu_flag_name;
468  const char *test_pattern;
469  int verbose;
470  int csv;
471  int tsv;
472  volatile sig_atomic_t catch_signals;
473 } state;
474 
475 /* PRNG state */
477 
478 /* float compare support code */
479 static int is_negative(union av_intfloat32 u)
480 {
481  return u.i >> 31;
482 }
483 
484 int float_near_ulp(float a, float b, unsigned max_ulp)
485 {
486  union av_intfloat32 x, y;
487 
488  x.f = a;
489  y.f = b;
490 
491  if (is_negative(x) != is_negative(y)) {
492  // handle -0.0 == +0.0
493  return a == b;
494  }
495 
496  if (llabs((int64_t)x.i - y.i) <= max_ulp)
497  return 1;
498 
499  return 0;
500 }
501 
502 int float_near_ulp_array(const float *a, const float *b, unsigned max_ulp,
503  unsigned len)
504 {
505  unsigned i;
506 
507  for (i = 0; i < len; i++) {
508  if (!float_near_ulp(a[i], b[i], max_ulp))
509  return 0;
510  }
511  return 1;
512 }
513 
514 int float_near_abs_eps(float a, float b, float eps)
515 {
516  float abs_diff = fabsf(a - b);
517  if (abs_diff < eps)
518  return 1;
519 
520  fprintf(stderr, "test failed comparing %g with %g (abs diff=%g with EPS=%g)\n", a, b, abs_diff, eps);
521 
522  return 0;
523 }
524 
525 int float_near_abs_eps_array(const float *a, const float *b, float eps,
526  unsigned len)
527 {
528  unsigned i;
529 
530  for (i = 0; i < len; i++) {
531  if (!float_near_abs_eps(a[i], b[i], eps))
532  return 0;
533  }
534  return 1;
535 }
536 
537 int float_near_abs_eps_ulp(float a, float b, float eps, unsigned max_ulp)
538 {
539  return float_near_ulp(a, b, max_ulp) || float_near_abs_eps(a, b, eps);
540 }
541 
542 int float_near_abs_eps_array_ulp(const float *a, const float *b, float eps,
543  unsigned max_ulp, unsigned len)
544 {
545  unsigned i;
546 
547  for (i = 0; i < len; i++) {
548  if (!float_near_abs_eps_ulp(a[i], b[i], eps, max_ulp))
549  return 0;
550  }
551  return 1;
552 }
553 
554 int double_near_abs_eps(double a, double b, double eps)
555 {
556  double abs_diff = fabs(a - b);
557 
558  return abs_diff < eps;
559 }
560 
561 int double_near_abs_eps_array(const double *a, const double *b, double eps,
562  unsigned len)
563 {
564  unsigned i;
565 
566  for (i = 0; i < len; i++) {
567  if (!double_near_abs_eps(a[i], b[i], eps))
568  return 0;
569  }
570  return 1;
571 }
572 
573 /* Print colored text to stderr if the terminal supports it */
574 static void color_printf(int color, const char *fmt, ...)
575 {
576  static int use_color = -1;
577  va_list arg;
578 
579 #if HAVE_SETCONSOLETEXTATTRIBUTE && HAVE_GETSTDHANDLE
580  static HANDLE con;
581  static WORD org_attributes;
582 
583  if (use_color < 0) {
584  CONSOLE_SCREEN_BUFFER_INFO con_info;
585  con = GetStdHandle(STD_ERROR_HANDLE);
586  if (con && con != INVALID_HANDLE_VALUE && GetConsoleScreenBufferInfo(con, &con_info)) {
587  org_attributes = con_info.wAttributes;
588  use_color = 1;
589  } else
590  use_color = 0;
591  }
592  if (use_color)
593  SetConsoleTextAttribute(con, (org_attributes & 0xfff0) | (color & 0x0f));
594 #else
595  if (use_color < 0) {
596  const char *term = getenv("TERM");
597  use_color = term && strcmp(term, "dumb") && isatty(2);
598  }
599  if (use_color)
600  fprintf(stderr, "\x1b[%d;3%dm", (color & 0x08) >> 3, color & 0x07);
601 #endif
602 
603  va_start(arg, fmt);
604  vfprintf(stderr, fmt, arg);
605  va_end(arg);
606 
607  if (use_color) {
608 #if HAVE_SETCONSOLETEXTATTRIBUTE && HAVE_GETSTDHANDLE
609  SetConsoleTextAttribute(con, org_attributes);
610 #else
611  fprintf(stderr, "\x1b[0m");
612 #endif
613  }
614 }
615 
616 /* Deallocate a tree */
618 {
619  if (f) {
620  CheckasmFuncVersion *v = f->versions.next;
621  while (v) {
622  CheckasmFuncVersion *next = v->next;
623  free(v);
624  v = next;
625  }
626 
627  destroy_func_tree(f->child[0]);
628  destroy_func_tree(f->child[1]);
629  free(f);
630  }
631 }
632 
633 /* Allocate a zero-initialized block, clean up and exit on failure */
634 static void *checkasm_malloc(size_t size)
635 {
636  void *ptr = calloc(1, size);
637  if (!ptr) {
638  fprintf(stderr, "checkasm: malloc failed\n");
639  destroy_func_tree(state.funcs);
640  exit(1);
641  }
642  return ptr;
643 }
644 
645 /* Get the suffix of the specified cpu flag */
646 static const char *cpu_suffix(int cpu)
647 {
648  int i = FF_ARRAY_ELEMS(cpus);
649 
650  while (--i >= 0)
651  if (cpu & cpus[i].flag)
652  return cpus[i].suffix;
653 
654  return "c";
655 }
656 
657 static int cmp_nop(const void *a, const void *b)
658 {
659  return *(const uint16_t*)a - *(const uint16_t*)b;
660 }
661 
662 /* Measure the overhead of the timing code (in decicycles) */
663 static int measure_nop_time(void)
664 {
665  uint16_t nops[10000];
666  int i, nop_sum = 0;
667  av_unused const int sysfd = state.sysfd;
668 
669  uint64_t t = 0;
670  for (i = 0; i < 10000; i++) {
671  PERF_START(t);
672  PERF_STOP(t);
673  nops[i] = t;
674  }
675 
676  qsort(nops, 10000, sizeof(uint16_t), cmp_nop);
677  for (i = 2500; i < 7500; i++)
678  nop_sum += nops[i];
679 
680  return nop_sum / 500;
681 }
682 
683 static inline double avg_cycles_per_call(const CheckasmPerf *const p)
684 {
685  if (p->iterations) {
686  const double cycles = (double)(10 * p->cycles) / p->iterations - state.nop_time;
687  if (cycles > 0.0)
688  return cycles / 32.0; /* 32 calls per iteration */
689  }
690  return 0.0;
691 }
692 
693 /* Print benchmark results */
695 {
696  if (f) {
697  CheckasmFuncVersion *v = &f->versions;
698  const CheckasmPerf *p = &v->perf;
699  const double baseline = avg_cycles_per_call(p);
700  double decicycles;
701 
702  print_benchs(f->child[0]);
703 
704  do {
705  if (p->iterations) {
706  p = &v->perf;
707  decicycles = avg_cycles_per_call(p);
708  if (state.csv || state.tsv) {
709  const char sep = state.csv ? ',' : '\t';
710  printf("%s%c%s%c%.1f\n", f->name, sep,
711  cpu_suffix(v->cpu), sep,
712  decicycles / 10.0);
713  } else {
714  const int pad_length = 10 + 50 -
715  printf("%s_%s:", f->name, cpu_suffix(v->cpu));
716  const double ratio = decicycles ?
717  baseline / decicycles : 0.0;
718  printf("%*.1f (%5.2fx)\n", FFMAX(pad_length, 0),
719  decicycles / 10.0, ratio);
720  }
721  }
722  } while ((v = v->next));
723 
724  print_benchs(f->child[1]);
725  }
726 }
727 
728 /* ASCIIbetical sort except preserving natural order for numbers */
729 static int cmp_func_names(const char *a, const char *b)
730 {
731  const char *start = a;
732  int ascii_diff, digit_diff;
733 
734  for (; !(ascii_diff = *(const unsigned char*)a - *(const unsigned char*)b) && *a; a++, b++);
735  for (; av_isdigit(*a) && av_isdigit(*b); a++, b++);
736 
737  if (a > start && av_isdigit(a[-1]) && (digit_diff = av_isdigit(*a) - av_isdigit(*b)))
738  return digit_diff;
739 
740  return ascii_diff;
741 }
742 
743 /* Perform a tree rotation in the specified direction and return the new root */
745 {
746  CheckasmFunc *r = f->child[dir^1];
747  f->child[dir^1] = r->child[dir];
748  r->child[dir] = f;
749  r->color = f->color;
750  f->color = 0;
751  return r;
752 }
753 
754 #define is_red(f) ((f) && !(f)->color)
755 
756 /* Balance a left-leaning red-black tree at the specified node */
757 static void balance_tree(CheckasmFunc **root)
758 {
759  CheckasmFunc *f = *root;
760 
761  if (is_red(f->child[0]) && is_red(f->child[1])) {
762  f->color ^= 1;
763  f->child[0]->color = f->child[1]->color = 1;
764  }
765 
766  if (!is_red(f->child[0]) && is_red(f->child[1]))
767  *root = rotate_tree(f, 0); /* Rotate left */
768  else if (is_red(f->child[0]) && is_red(f->child[0]->child[0]))
769  *root = rotate_tree(f, 1); /* Rotate right */
770 }
771 
772 /* Get a node with the specified name, creating it if it doesn't exist */
773 static CheckasmFunc *get_func(CheckasmFunc **root, const char *name)
774 {
775  CheckasmFunc *f = *root;
776 
777  if (f) {
778  /* Search the tree for a matching node */
779  int cmp = cmp_func_names(name, f->name);
780  if (cmp) {
781  f = get_func(&f->child[cmp > 0], name);
782 
783  /* Rebalance the tree on the way up if a new node was inserted */
784  if (!f->versions.func)
785  balance_tree(root);
786  }
787  } else {
788  /* Allocate and insert a new node into the tree */
789  int name_length = strlen(name);
790  f = *root = checkasm_malloc(sizeof(CheckasmFunc) + name_length);
791  memcpy(f->name, name, name_length + 1);
792  }
793 
794  return f;
795 }
796 
798 
799 /* Crash handling: attempt to catch crashes and handle them
800  * gracefully instead of just aborting abruptly. */
801 #ifdef _WIN32
802 #if WINAPI_FAMILY_PARTITION(WINAPI_PARTITION_DESKTOP)
803 static LONG NTAPI signal_handler(EXCEPTION_POINTERS *e) {
804  int s;
805 
806  if (!state.catch_signals)
807  return EXCEPTION_CONTINUE_SEARCH;
808 
809  switch (e->ExceptionRecord->ExceptionCode) {
810  case EXCEPTION_FLT_DIVIDE_BY_ZERO:
811  case EXCEPTION_INT_DIVIDE_BY_ZERO:
812  s = SIGFPE;
813  break;
814  case EXCEPTION_ILLEGAL_INSTRUCTION:
815  case EXCEPTION_PRIV_INSTRUCTION:
816  s = SIGILL;
817  break;
818  case EXCEPTION_ACCESS_VIOLATION:
819  case EXCEPTION_ARRAY_BOUNDS_EXCEEDED:
820  case EXCEPTION_DATATYPE_MISALIGNMENT:
821  case EXCEPTION_STACK_OVERFLOW:
822  s = SIGSEGV;
823  break;
824  case EXCEPTION_IN_PAGE_ERROR:
825  s = SIGBUS;
826  break;
827  default:
828  return EXCEPTION_CONTINUE_SEARCH;
829  }
830  state.catch_signals = 0;
832  return EXCEPTION_CONTINUE_EXECUTION; /* never reached, but shuts up gcc */
833 }
834 #endif
835 #elif !defined(_WASI_EMULATED_SIGNAL)
836 static void signal_handler(int s);
837 
838 static const struct sigaction signal_handler_act = {
839  .sa_handler = signal_handler,
840  .sa_flags = SA_RESETHAND,
841 };
842 
843 static void signal_handler(int s) {
844  if (state.catch_signals) {
845  state.catch_signals = 0;
846  sigaction(s, &signal_handler_act, NULL);
848  }
849 }
850 #endif
851 
852 /* Compares a string with a wildcard pattern. */
853 static int wildstrcmp(const char *str, const char *pattern)
854 {
855  const char *wild = strchr(pattern, '*');
856  if (wild) {
857  const size_t len = wild - pattern;
858  if (strncmp(str, pattern, len)) return 1;
859  while (*++wild == '*');
860  if (!*wild) return 0;
861  str += len;
862  while (*str && wildstrcmp(str, wild)) str++;
863  return !*str;
864  }
865  return strcmp(str, pattern);
866 }
867 
868 /* Perform tests and benchmarks for the specified cpu flag if supported by the host */
869 static void check_cpu_flag(const char *name, int flag)
870 {
871  int old_cpu_flag = state.cpu_flag;
872 
873  flag |= old_cpu_flag;
874  av_force_cpu_flags(-1);
875  state.cpu_flag = flag & av_get_cpu_flags();
876  av_force_cpu_flags(state.cpu_flag);
877 
878  if (!flag || state.cpu_flag != old_cpu_flag) {
879  int i;
880 
881  state.cpu_flag_name = name;
882  for (i = 0; tests[i].func; i++) {
883  if (state.test_pattern && wildstrcmp(tests[i].name, state.test_pattern))
884  continue;
885  state.current_test_name = tests[i].name;
886  tests[i].func();
887  }
888  }
889 }
890 
891 /* Print the name of the current CPU flag, but only do it once */
892 static void print_cpu_name(void)
893 {
894  if (state.cpu_flag_name) {
895  color_printf(COLOR_YELLOW, "%s:\n", state.cpu_flag_name);
896  state.cpu_flag_name = NULL;
897  }
898 }
899 
900 #if CONFIG_LINUX_PERF
901 static int bench_init_linux(void)
902 {
903  struct perf_event_attr attr = {
904  .type = PERF_TYPE_HARDWARE,
905  .size = sizeof(struct perf_event_attr),
906  .config = PERF_COUNT_HW_CPU_CYCLES,
907  .disabled = 1, // start counting only on demand
908  .exclude_kernel = 1,
909  .exclude_hv = 1,
910 #if !ARCH_X86
911  .exclude_guest = 1,
912 #endif
913  };
914 
915  fprintf(stderr, "benchmarking with Linux Perf Monitoring API\n");
916 
917  state.sysfd = syscall(__NR_perf_event_open, &attr, 0, -1, -1, 0);
918  if (state.sysfd == -1) {
919  perror("perf_event_open");
920  return -1;
921  }
922  return 0;
923 }
924 #elif CONFIG_MACOS_KPERF
925 static int bench_init_kperf(void)
926 {
927  ff_kperf_init();
928  return 0;
929 }
930 #else
931 static int bench_init_ffmpeg(void)
932 {
933 #ifdef AV_READ_TIME
934  if (!checkasm_save_context()) {
936  AV_READ_TIME();
938  } else {
939  fprintf(stderr, "checkasm: unable to execute platform specific timer\n");
940  return -1;
941  }
942  fprintf(stderr, "benchmarking with native FFmpeg timers\n");
943  return 0;
944 #else
945  fprintf(stderr, "checkasm: --bench is not supported on your system\n");
946  return -1;
947 #endif
948 }
949 #endif
950 
951 static int bench_init(void)
952 {
953 #if CONFIG_LINUX_PERF
954  int ret = bench_init_linux();
955 #elif CONFIG_MACOS_KPERF
956  int ret = bench_init_kperf();
957 #else
958  int ret = bench_init_ffmpeg();
959 #endif
960  if (ret < 0)
961  return ret;
962 
963  state.nop_time = measure_nop_time();
964  fprintf(stderr, "nop: %d.%d\n", state.nop_time/10, state.nop_time%10);
965  return 0;
966 }
967 
968 static void bench_uninit(void)
969 {
970 #if CONFIG_LINUX_PERF
971  close(state.sysfd);
972 #endif
973 }
974 
975 static int usage(const char *path)
976 {
977  fprintf(stderr,
978  "Usage: %s [options...] [seed]\n"
979  " --test=<pattern> Run specific test.\n"
980  " --bench Run benchmark.\n"
981  " --csv, --tsv Output results in rows of comma or tab separated values.\n"
982  " --runs=<ptwo> Manual number of benchmark iterations to run 2**<ptwo>.\n"
983  " --verbose Increase verbosity.\n",
984  path);
985  return 1;
986 }
987 
988 int main(int argc, char *argv[])
989 {
990  unsigned int seed = av_get_random_seed();
991  int i, ret = 0;
992  char arch_info_buf[50] = "";
993 
994 #ifdef _WIN32
995 #if WINAPI_FAMILY_PARTITION(WINAPI_PARTITION_DESKTOP)
996  AddVectoredExceptionHandler(0, signal_handler);
997 #endif
998 #elif !defined(_WASI_EMULATED_SIGNAL)
999  sigaction(SIGBUS, &signal_handler_act, NULL);
1000  sigaction(SIGFPE, &signal_handler_act, NULL);
1001  sigaction(SIGILL, &signal_handler_act, NULL);
1002  sigaction(SIGSEGV, &signal_handler_act, NULL);
1003 #endif
1004 #if HAVE_PRCTL && defined(PR_SET_UNALIGN)
1005  prctl(PR_SET_UNALIGN, PR_UNALIGN_SIGBUS);
1006 #endif
1007 #if ARCH_ARM && HAVE_ARMV5TE_EXTERNAL
1009  checkasm_checked_call = checkasm_checked_call_vfp;
1010 #endif
1011 
1012  if (!tests[0].func || !cpus[0].flag) {
1013  fprintf(stderr, "checkasm: no tests to perform\n");
1014  return 0;
1015  }
1016 
1017  for (i = 1; i < argc; i++) {
1018  const char *arg = argv[i];
1019  unsigned long l;
1020  char *end;
1021 
1022  if (!strncmp(arg, "--bench", 7)) {
1023  if (bench_init() < 0)
1024  return 1;
1025  if (arg[7] == '=') {
1026  state.bench_pattern = arg + 8;
1027  state.bench_pattern_len = strlen(state.bench_pattern);
1028  } else
1029  state.bench_pattern = "*";
1030  } else if (!strncmp(arg, "--test=", 7)) {
1031  state.test_pattern = arg + 7;
1032  } else if (!strcmp(arg, "--csv")) {
1033  state.csv = 1; state.tsv = 0;
1034  } else if (!strcmp(arg, "--tsv")) {
1035  state.csv = 0; state.tsv = 1;
1036  } else if (!strcmp(arg, "--verbose") || !strcmp(arg, "-v")) {
1037  state.verbose = 1;
1038  } else if (!strncmp(arg, "--runs=", 7)) {
1039  l = strtoul(arg + 7, &end, 10);
1040  if (*end == '\0') {
1041  if (l > 30) {
1042  fprintf(stderr, "checkasm: error: runs exponent must be within the range 0 <= 30\n");
1043  usage(argv[0]);
1044  }
1045  bench_runs = 1U << l;
1046  } else {
1047  return usage(argv[0]);
1048  }
1049  } else if ((l = strtoul(arg, &end, 10)) <= UINT_MAX &&
1050  *end == '\0') {
1051  seed = l;
1052  } else {
1053  return usage(argv[0]);
1054  }
1055  }
1056 
1057 #if ARCH_AARCH64 && HAVE_SVE
1058  if (have_sve(av_get_cpu_flags()))
1059  snprintf(arch_info_buf, sizeof(arch_info_buf),
1060  "SVE %d bits, ", 8 * ff_aarch64_sve_length());
1061 #endif
1062 #if ARCH_AARCH64 && HAVE_SME
1063  if (have_sme(av_get_cpu_flags()))
1064  av_strlcatf(arch_info_buf, sizeof(arch_info_buf),
1065  "SME %d bits, ", 8 * ff_aarch64_sme_length());
1066 #endif
1067 #if ARCH_RISCV && HAVE_RVV
1069  snprintf(arch_info_buf, sizeof (arch_info_buf),
1070  "%zu-bit vectors, ", 8 * ff_get_rv_vlenb());
1071 #endif
1072  fprintf(stderr, "checkasm: %susing random seed %u\n", arch_info_buf, seed);
1074 
1075  if (state.bench_pattern)
1076  fprintf(stderr, "checkasm: bench runs %" PRIu64 " (1 << %i)\n", bench_runs, av_log2(bench_runs));
1077 
1078  check_cpu_flag(NULL, 0);
1079  for (i = 0; cpus[i].flag; i++)
1081 
1082  if (state.num_failed) {
1083  fprintf(stderr, "checkasm: %d of %d tests have failed\n", state.num_failed, state.num_checked);
1084  ret = 1;
1085  } else {
1086  fprintf(stderr, "checkasm: all %d tests passed\n", state.num_checked);
1087  if (state.bench_pattern) {
1088  print_benchs(state.funcs);
1089  }
1090  }
1091 
1092  destroy_func_tree(state.funcs);
1093  bench_uninit();
1094  return ret;
1095 }
1096 
1097 /* Decide whether or not the specified function needs to be tested and
1098  * allocate/initialize data structures if needed. Returns a pointer to a
1099  * reference function if the function should be tested, otherwise NULL */
1100 void *checkasm_check_func(void *func, const char *name, ...)
1101 {
1102  char name_buf[256];
1103  void *ref = func;
1105  int name_length;
1106  va_list arg;
1107 
1108  va_start(arg, name);
1109  name_length = vsnprintf(name_buf, sizeof(name_buf), name, arg);
1110  va_end(arg);
1111 
1112  if (!func || name_length <= 0 || name_length >= sizeof(name_buf))
1113  return NULL;
1114 
1115  state.current_func = get_func(&state.funcs, name_buf);
1116  state.funcs->color = 1;
1117  v = &state.current_func->versions;
1118 
1119  if (v->func) {
1120  CheckasmFuncVersion *prev;
1121  do {
1122  /* Only test functions that haven't already been tested */
1123  if (v->func == func)
1124  return NULL;
1125 
1126  if (v->ok)
1127  ref = v->func;
1128 
1129  prev = v;
1130  } while ((v = v->next));
1131 
1132  v = prev->next = checkasm_malloc(sizeof(CheckasmFuncVersion));
1133  }
1134 
1135  v->func = func;
1136  v->ok = 1;
1137  v->cpu = state.cpu_flag;
1138  state.current_func_ver = v;
1139 
1140  if (state.cpu_flag)
1141  state.num_checked++;
1142 
1143  return ref;
1144 }
1145 
1146 /* Decide whether or not the current function needs to be benchmarked */
1148 {
1149  return !state.num_failed && state.bench_pattern &&
1150  !wildstrcmp(state.current_func->name, state.bench_pattern);
1151 }
1152 
1153 /* Indicate that the current test has failed, return whether verbose printing
1154  * is requested. */
1155 int checkasm_fail_func(const char *msg, ...)
1156 {
1157  if (state.current_func_ver && state.current_func_ver->cpu &&
1158  state.current_func_ver->ok)
1159  {
1160  va_list arg;
1161 
1162  print_cpu_name();
1163  fprintf(stderr, " %s_%s (", state.current_func->name, cpu_suffix(state.current_func_ver->cpu));
1164  va_start(arg, msg);
1165  vfprintf(stderr, msg, arg);
1166  va_end(arg);
1167  fprintf(stderr, ")\n");
1168 
1169  state.current_func_ver->ok = 0;
1170  state.num_failed++;
1171  }
1172  return state.verbose;
1173 }
1174 
1176  state.catch_signals = enabled;
1177 }
1178 
1180  if (s) {
1181 #ifdef __GLIBC__
1182  checkasm_fail_func("fatal signal %d: %s", s, strsignal(s));
1183 #else
1184  checkasm_fail_func(s == SIGFPE ? "fatal arithmetic error" :
1185  s == SIGILL ? "illegal instruction" :
1186  s == SIGBUS ? "bus error" :
1187  "segmentation fault");
1188 #endif
1189  }
1190  return s;
1191 }
1192 
1193 /* Get the benchmark context of the current function */
1195 {
1196  CheckasmPerf *perf = &state.current_func_ver->perf;
1197  memset(perf, 0, sizeof(*perf));
1198  perf->sysfd = state.sysfd;
1199  return perf;
1200 }
1201 
1202 /* Print the outcome of all tests performed since the last time this function was called */
1203 void checkasm_report(const char *name, ...)
1204 {
1205  static int prev_checked, prev_failed, max_length;
1206 
1207  if (state.num_checked > prev_checked) {
1208  int pad_length = max_length + 4;
1209  va_list arg;
1210 
1211  print_cpu_name();
1212  pad_length -= fprintf(stderr, " - %s.", state.current_test_name);
1213  va_start(arg, name);
1214  pad_length -= vfprintf(stderr, name, arg);
1215  va_end(arg);
1216  fprintf(stderr, "%*c", FFMAX(pad_length, 0) + 2, '[');
1217 
1218  if (state.num_failed == prev_failed)
1219  color_printf(COLOR_GREEN, "OK");
1220  else
1221  color_printf(COLOR_RED, "FAILED");
1222  fprintf(stderr, "]\n");
1223 
1224  prev_checked = state.num_checked;
1225  prev_failed = state.num_failed;
1226  } else if (!state.cpu_flag) {
1227  /* Calculate the amount of padding required to make the output vertically aligned */
1228  int length = strlen(state.current_test_name);
1229  va_list arg;
1230 
1231  va_start(arg, name);
1232  length += vsnprintf(NULL, 0, name, arg);
1233  va_end(arg);
1234 
1235  if (length > max_length)
1236  max_length = length;
1237  }
1238 }
1239 
1240 static int check_err(const char *file, int line,
1241  const char *name, int w, int h,
1242  int *err)
1243 {
1244  if (*err)
1245  return 0;
1246  if (!checkasm_fail_func("%s:%d", file, line))
1247  return 1;
1248  *err = 1;
1249  fprintf(stderr, "%s (%dx%d):\n", name, w, h);
1250  return 0;
1251 }
1252 
1253 #define DEF_CHECKASM_CHECK_BODY(compare, type, fmt) \
1254 do { \
1255  int64_t aligned_w = (w - 1LL + align_w) & ~(align_w - 1); \
1256  int64_t aligned_h = (h - 1LL + align_h) & ~(align_h - 1); \
1257  int err = 0; \
1258  int y = 0; \
1259  av_assert0(aligned_w == (int32_t)aligned_w);\
1260  av_assert0(aligned_h == (int32_t)aligned_h);\
1261  stride1 /= sizeof(*buf1); \
1262  stride2 /= sizeof(*buf2); \
1263  for (y = 0; y < h; y++) \
1264  if (!compare(&buf1[y*stride1], &buf2[y*stride2], w)) \
1265  break; \
1266  if (y != h) { \
1267  if (check_err(file, line, name, w, h, &err)) \
1268  return 1; \
1269  for (y = 0; y < h; y++) { \
1270  for (int x = 0; x < w; x++) \
1271  fprintf(stderr, " " fmt, buf1[x]); \
1272  fprintf(stderr, " "); \
1273  for (int x = 0; x < w; x++) \
1274  fprintf(stderr, " " fmt, buf2[x]); \
1275  fprintf(stderr, " "); \
1276  for (int x = 0; x < w; x++) \
1277  fprintf(stderr, "%c", buf1[x] != buf2[x] ? 'x' : '.'); \
1278  buf1 += stride1; \
1279  buf2 += stride2; \
1280  fprintf(stderr, "\n"); \
1281  } \
1282  buf1 -= h*stride1; \
1283  buf2 -= h*stride2; \
1284  } \
1285  for (y = -padding; y < 0; y++) \
1286  if (!compare(&buf1[y*stride1 - padding], &buf2[y*stride2 - padding], \
1287  w + 2*padding)) { \
1288  if (check_err(file, line, name, w, h, &err)) \
1289  return 1; \
1290  fprintf(stderr, " overwrite above\n"); \
1291  break; \
1292  } \
1293  for (y = aligned_h; y < aligned_h + padding; y++) \
1294  if (!compare(&buf1[y*stride1 - padding], &buf2[y*stride2 - padding], \
1295  w + 2*padding)) { \
1296  if (check_err(file, line, name, w, h, &err)) \
1297  return 1; \
1298  fprintf(stderr, " overwrite below\n"); \
1299  break; \
1300  } \
1301  for (y = 0; y < h; y++) \
1302  if (!compare(&buf1[y*stride1 - padding], &buf2[y*stride2 - padding], \
1303  padding)) { \
1304  if (check_err(file, line, name, w, h, &err)) \
1305  return 1; \
1306  fprintf(stderr, " overwrite left\n"); \
1307  break; \
1308  } \
1309  for (y = 0; y < h; y++) \
1310  if (!compare(&buf1[y*stride1 + aligned_w], &buf2[y*stride2 + aligned_w], \
1311  padding)) { \
1312  if (check_err(file, line, name, w, h, &err)) \
1313  return 1; \
1314  fprintf(stderr, " overwrite right\n"); \
1315  break; \
1316  } \
1317  return err; \
1318 } while (0)
1319 
1320 #define cmp_int(a, b, len) (!memcmp(a, b, (len) * sizeof(*(a))))
1321 #define DEF_CHECKASM_CHECK_FUNC(type, fmt) \
1322 int checkasm_check_##type(const char *file, int line, \
1323  const type *buf1, ptrdiff_t stride1, \
1324  const type *buf2, ptrdiff_t stride2, \
1325  int w, int h, const char *name, \
1326  int align_w, int align_h, \
1327  int padding) \
1328 { \
1329  DEF_CHECKASM_CHECK_BODY(cmp_int, type, fmt); \
1330 }
1331 
1332 DEF_CHECKASM_CHECK_FUNC(uint8_t, "%02x")
1333 DEF_CHECKASM_CHECK_FUNC(uint16_t, "%04x")
1334 DEF_CHECKASM_CHECK_FUNC(uint32_t, "%08x")
1335 DEF_CHECKASM_CHECK_FUNC(int16_t, "%6d")
1337 
1338 int checkasm_check_float_ulp(const char *file, int line,
1339  const float *buf1, ptrdiff_t stride1,
1340  const float *buf2, ptrdiff_t stride2,
1341  int w, int h, const char *name,
1342  unsigned max_ulp, int align_w, int align_h,
1343  int padding)
1344 {
1345  #define cmp_float(a, b, len) float_near_ulp_array(a, b, max_ulp, len)
1346  DEF_CHECKASM_CHECK_BODY(cmp_float, float, "%g");
1347  #undef cmp_float
1348 }
checkasm_check_nlmeans
void checkasm_check_nlmeans(void)
Definition: vf_nlmeans.c:34
AV_CPU_FLAG_VFP
#define AV_CPU_FLAG_VFP
Definition: cpu.h:71
checkasm_check_sw_gbrp
void checkasm_check_sw_gbrp(void)
Definition: sw_gbrp.c:398
bench_pattern_len
int bench_pattern_len
Definition: checkasm.c:458
av_force_cpu_flags
void av_force_cpu_flags(int arg)
Disables cpu detection and forces the specified flags.
Definition: cpu.c:81
checkasm_check_vp8dsp
void checkasm_check_vp8dsp(void)
Definition: vp8dsp.c:511
checkasm_check_idet
void checkasm_check_idet(void)
Definition: vf_idet.c:58
checkasm_check_blockdsp
void checkasm_check_blockdsp(void)
Definition: blockdsp.c:78
AV_CPU_FLAG_SSE3
#define AV_CPU_FLAG_SSE3
Prescott SSE3 functions.
Definition: cpu.h:41
checkasm_check_fdctdsp
void checkasm_check_fdctdsp(void)
Definition: fdctdsp.c:67
destroy_func_tree
static void destroy_func_tree(CheckasmFunc *f)
Definition: checkasm.c:617
r
const char * r
Definition: vf_curves.c:127
checkasm_check_sw_xyz2rgb
void checkasm_check_sw_xyz2rgb(void)
Definition: sw_xyz2rgb.c:104
printf
__device__ int printf(const char *,...)
checkasm_check_videodsp
void checkasm_check_videodsp(void)
Definition: videodsp.c:80
cmp_nop
static int cmp_nop(const void *a, const void *b)
Definition: checkasm.c:657
checkasm_check_vf_eq
void checkasm_check_vf_eq(void)
Definition: vf_eq.c:76
AV_CPU_FLAG_RVB_BASIC
#define AV_CPU_FLAG_RVB_BASIC
Basic bit-manipulations.
Definition: cpu.h:103
COLOR_RED
#define COLOR_RED
Definition: checkasm.c:85
checkasm_check_dcadsp
void checkasm_check_dcadsp(void)
Definition: dcadsp.c:89
wildstrcmp
static int wildstrcmp(const char *str, const char *pattern)
Definition: checkasm.c:853
checkasm_lfg
AVLFG checkasm_lfg
Definition: checkasm.c:476
color
Definition: vf_paletteuse.c:513
av_lfg_init
av_cold void av_lfg_init(AVLFG *c, unsigned int seed)
Definition: lfg.c:32
checkasm_check_alacdsp
void checkasm_check_alacdsp(void)
Definition: alacdsp.c:116
AV_CPU_FLAG_LSX
#define AV_CPU_FLAG_LSX
Definition: cpu.h:90
checkasm_check_v210dec
void checkasm_check_v210dec(void)
Definition: v210dec.c:47
float_near_abs_eps_array
int float_near_abs_eps_array(const float *a, const float *b, float eps, unsigned len)
Definition: checkasm.c:525
AV_CPU_FLAG_SSE3SLOW
#define AV_CPU_FLAG_SSE3SLOW
SSE3 supported, but usually not faster.
Definition: cpu.h:42
CheckasmFunc
Definition: checkasm.c:444
checkasm_check_sw_yuv2yuv
void checkasm_check_sw_yuv2yuv(void)
Definition: sw_yuv2yuv.c:131
name
const char * name
Definition: checkasm.c:115
checkasm_fail_func
int checkasm_fail_func(const char *msg,...)
Definition: checkasm.c:1155
checkasm_check_png
void checkasm_check_png(void)
Definition: png.c:106
bench_pattern
const char * bench_pattern
Definition: checkasm.c:457
AV_CPU_FLAG_RV_ZVBB
#define AV_CPU_FLAG_RV_ZVBB
Vector basic bit-manipulations.
Definition: cpu.h:107
int64_t
long long int64_t
Definition: coverity.c:34
av_unused
#define av_unused
Definition: attributes.h:156
checkasm_check_hevc_idct
void checkasm_check_hevc_idct(void)
Definition: hevc_idct.c:108
checkasm_check_sw_scale
void checkasm_check_sw_scale(void)
Definition: sw_scale.c:460
CheckasmFunc::child
struct CheckasmFunc * child[2]
Definition: checkasm.c:445
AV_CPU_FLAG_SVE2
#define AV_CPU_FLAG_SVE2
Definition: cpu.h:79
checkasm_check_afir
void checkasm_check_afir(void)
Definition: af_afir.c:144
checkasm_check_aes
void checkasm_check_aes(void)
Definition: aes.c:28
print_cpu_name
static void print_cpu_name(void)
Definition: checkasm.c:892
u
#define u(width, name, range_min, range_max)
Definition: cbs_apv.c:68
state
static struct @560 state
av_intfloat32::i
uint32_t i
Definition: intfloat.h:28
AV_CPU_FLAG_SVE
#define AV_CPU_FLAG_SVE
Definition: cpu.h:78
float_near_abs_eps
int float_near_abs_eps(float a, float b, float eps)
Definition: checkasm.c:514
b
#define b
Definition: input.c:42
checkasm_check_aacpsdsp
void checkasm_check_aacpsdsp(void)
Definition: aacpsdsp.c:234
bench_uninit
static void bench_uninit(void)
Definition: checkasm.c:968
CheckasmFuncVersion::cpu
int cpu
Definition: checkasm.c:439
cpu_flag
int cpu_flag
Definition: checkasm.c:466
AV_CPU_FLAG_DOTPROD
#define AV_CPU_FLAG_DOTPROD
Definition: cpu.h:76
checkasm_check_vvc_sao
void checkasm_check_vvc_sao(void)
Definition: vvc_sao.c:134
FFMAX
#define FFMAX(a, b)
Definition: macros.h:47
av_get_cpu_flags
int av_get_cpu_flags(void)
Return the flags which specify extensions supported by the CPU.
Definition: cpu.c:109
cpu_suffix
static const char * cpu_suffix(int cpu)
Definition: checkasm.c:646
CheckasmPerf::sysfd
int sysfd
Definition: checkasm.h:348
intfloat.h
checkasm_check_h264dsp
void checkasm_check_h264dsp(void)
Definition: h264dsp.c:503
av_strlcatf
size_t av_strlcatf(char *dst, size_t size, const char *fmt,...)
Definition: avstring.c:103
tf_sess_config.config
config
Definition: tf_sess_config.py:33
color_printf
static void color_printf(int color, const char *fmt,...)
Definition: checkasm.c:574
AV_CPU_FLAG_RVV_F64
#define AV_CPU_FLAG_RVV_F64
Vectors of double's.
Definition: cpu.h:102
checkasm_check_vp9_ipred
void checkasm_check_vp9_ipred(void)
Definition: vp9dsp.c:50
av_get_random_seed
uint32_t av_get_random_seed(void)
Get a seed to use in conjunction with random functions.
Definition: random_seed.c:196
main
int main(int argc, char *argv[])
Definition: checkasm.c:988
checkasm_check_float_ulp
int checkasm_check_float_ulp(const char *file, int line, const float *buf1, ptrdiff_t stride1, const float *buf2, ptrdiff_t stride2, int w, int h, const char *name, unsigned max_ulp, int align_w, int align_h, int padding)
Definition: checkasm.c:1338
checkasm_check_motion
void checkasm_check_motion(void)
Definition: motion.c:137
close
static av_cold void close(AVCodecParserContext *s)
Definition: apv_parser.c:197
checkasm_check_h263dsp
void checkasm_check_h263dsp(void)
Definition: h263dsp.c:54
AV_CPU_FLAG_RVB
#define AV_CPU_FLAG_RVB
B (bit manipulations)
Definition: cpu.h:109
AV_CPU_FLAG_SSSE3
#define AV_CPU_FLAG_SSSE3
Conroe SSSE3 functions.
Definition: cpu.h:44
double_near_abs_eps
int double_near_abs_eps(double a, double b, double eps)
Definition: checkasm.c:554
CheckasmPerf
Definition: checkasm.h:347
current_test_name
const char * current_test_name
Definition: checkasm.c:456
PERF_START
#define PERF_START(t)
Definition: checkasm.h:420
rotate_tree
static CheckasmFunc * rotate_tree(CheckasmFunc *f, int dir)
Definition: checkasm.c:744
dummy
int dummy
Definition: motion.c:64
AV_CPU_FLAG_XOP
#define AV_CPU_FLAG_XOP
Bulldozer XOP functions.
Definition: cpu.h:53
checkasm_check_vp9dsp
void checkasm_check_vp9dsp(void)
Definition: vp9dsp.c:627
checkasm.h
checkasm_check_vf_threshold
void checkasm_check_vf_threshold(void)
Definition: vf_threshold.c:79
checkasm_check_vf_gblur
void checkasm_check_vf_gblur(void)
Definition: vf_gblur.c:82
float_near_abs_eps_ulp
int float_near_abs_eps_ulp(float a, float b, float eps, unsigned max_ulp)
Definition: checkasm.c:537
have_sve
#define have_sve(flags)
Definition: cpu.h:31
checkasm_check_vvc_mc
void checkasm_check_vvc_mc(void)
Definition: vvc_mc.c:471
AV_CPU_FLAG_VSX
#define AV_CPU_FLAG_VSX
ISA 2.06.
Definition: cpu.h:65
fabsf
static __device__ float fabsf(float a)
Definition: cuda_runtime.h:181
checkasm_check_sw_yuv2rgb
void checkasm_check_sw_yuv2rgb(void)
Definition: sw_yuv2rgb.c:235
AV_CPU_FLAG_RVV_F32
#define AV_CPU_FLAG_RVV_F32
Vectors of float's *‍/.
Definition: cpu.h:100
num_failed
int num_failed
Definition: checkasm.c:460
checkasm_check_float_dsp
void checkasm_check_float_dsp(void)
Definition: float_dsp.c:297
AV_CPU_FLAG_AVX512
#define AV_CPU_FLAG_AVX512
AVX-512 functions: requires OS support even if YMM/ZMM registers aren't used.
Definition: cpu.h:60
checkasm_check_hevc_dequant
void checkasm_check_hevc_dequant(void)
Definition: hevc_dequant.c:65
bench_init
static int bench_init(void)
Definition: checkasm.c:951
avassert.h
checkasm_check_pixblockdsp
void checkasm_check_pixblockdsp(void)
Definition: pixblockdsp.c:82
FF_ARRAY_ELEMS
#define FF_ARRAY_ELEMS(a)
Definition: sinewin_tablegen.c:29
CheckasmFuncVersion
Definition: checkasm.c:435
COLOR_YELLOW
#define COLOR_YELLOW
Definition: checkasm.c:87
nop_time
int nop_time
Definition: checkasm.c:463
CheckasmFuncVersion::perf
CheckasmPerf perf
Definition: checkasm.c:440
s
#define s(width, name)
Definition: cbs_vp9.c:198
checkasm_check_vf_bwdif
void checkasm_check_vf_bwdif(void)
Definition: vf_bwdif.c:72
checkasm_report
void checkasm_report(const char *name,...)
Definition: checkasm.c:1203
get_func
static CheckasmFunc * get_func(CheckasmFunc **root, const char *name)
Definition: checkasm.c:773
cpu.h
checkasm_check_sw_rgb
void checkasm_check_sw_rgb(void)
Definition: sw_rgb.c:804
checkasm_check_hevc_sao
void checkasm_check_hevc_sao(void)
Definition: hevc_sao.c:136
num_checked
int num_checked
Definition: checkasm.c:459
AV_CPU_FLAG_ARMV6
#define AV_CPU_FLAG_ARMV6
Definition: cpu.h:69
AV_READ_TIME
#define AV_READ_TIME
Definition: timer.h:28
AV_CPU_FLAG_SSE4
#define AV_CPU_FLAG_SSE4
Penryn SSE4.1 functions.
Definition: cpu.h:47
AV_CPU_FLAG_SME2
#define AV_CPU_FLAG_SME2
Definition: cpu.h:82
checkasm_check_hevc_pel
void checkasm_check_hevc_pel(void)
Definition: hevc_pel.c:621
checkasm_set_signal_handler_state
void checkasm_set_signal_handler_state(int enabled)
Definition: checkasm.c:1175
CheckasmFuncVersion::ok
int ok
Definition: checkasm.c:438
arg
const char * arg
Definition: jacosubdec.c:65
func
void(* func)(void)
Definition: checkasm.c:116
cmp_float
#define cmp_float(a, b, len)
fabs
static __device__ float fabs(float a)
Definition: cuda_runtime.h:182
NULL
#define NULL
Definition: coverity.c:32
cpus
static const struct @559 cpus[]
measure_nop_time
static int measure_nop_time(void)
Definition: checkasm.c:663
checkasm_check_takdsp
void checkasm_check_takdsp(void)
Definition: takdsp.c:145
double_near_abs_eps_array
int double_near_abs_eps_array(const double *a, const double *b, double eps, unsigned len)
Definition: checkasm.c:561
checkasm_check_sw_ops
void checkasm_check_sw_ops(void)
Definition: sw_ops.c:750
AV_CPU_FLAG_LASX
#define AV_CPU_FLAG_LASX
Definition: cpu.h:91
AV_CPU_FLAG_AVX512ICL
#define AV_CPU_FLAG_AVX512ICL
F/CD/BW/DQ/VL/VNNI/IFMA/VBMI/VBMI2/VPOPCNTDQ/BITALG/GFNI/VAES/VPCLMULQDQ.
Definition: cpu.h:61
checkasm_check_fmtconvert
void checkasm_check_fmtconvert(void)
Definition: fmtconvert.c:46
checkasm_load_context
#define checkasm_load_context(s)
Definition: checkasm.h:77
AV_CPU_FLAG_MSA
#define AV_CPU_FLAG_MSA
Definition: cpu.h:87
checkasm_save_context
#define checkasm_save_context()
Definition: checkasm.h:76
COLOR_GREEN
#define COLOR_GREEN
Definition: checkasm.c:86
checkasm_check_aacencdsp
void checkasm_check_aacencdsp(void)
Definition: aacencdsp.c:100
checkasm_check_vorbisdsp
void checkasm_check_vorbisdsp(void)
Definition: vorbisdsp.c:76
cpu.h
double
double
Definition: af_crystalizer.c:132
av_intfloat32
Definition: intfloat.h:27
checkasm_check_cavsdsp
void checkasm_check_cavsdsp(void)
Definition: cavsdsp.c:116
checkasm_check_apv_dsp
void checkasm_check_apv_dsp(void)
Definition: apv_dsp.c:96
AV_CPU_FLAG_CMOV
#define AV_CPU_FLAG_CMOV
supports cmov instruction
Definition: cpu.h:55
seed
static unsigned int seed
Definition: videogen.c:78
checkasm_check_hpeldsp
void checkasm_check_hpeldsp(void)
Definition: hpeldsp.c:54
AV_CPU_FLAG_ALTIVEC
#define AV_CPU_FLAG_ALTIVEC
standard
Definition: cpu.h:64
checkasm_check_rv40dsp
void checkasm_check_rv40dsp(void)
Definition: rv40dsp.c:71
print_benchs
static void print_benchs(CheckasmFunc *f)
Definition: checkasm.c:694
current_func
CheckasmFunc * current_func
Definition: checkasm.c:454
AV_CPU_FLAG_I8MM
#define AV_CPU_FLAG_I8MM
Definition: cpu.h:77
checkasm_check_vc1dsp
void checkasm_check_vc1dsp(void)
Definition: vc1dsp.c:476
catch_signals
volatile sig_atomic_t catch_signals
Definition: checkasm.c:472
checkasm_check_h264chroma
void checkasm_check_h264chroma(void)
Definition: h264chroma.c:82
AV_CPU_FLAG_SSE2
#define AV_CPU_FLAG_SSE2
PIV SSE2 functions.
Definition: cpu.h:37
AV_CPU_FLAG_CLMUL
#define AV_CPU_FLAG_CLMUL
Carry-less Multiplication instruction.
Definition: cpu.h:50
CheckasmFunc::color
uint8_t color
Definition: checkasm.c:447
AVLFG
Context structure for the Lagged Fibonacci PRNG.
Definition: lfg.h:33
f
f
Definition: af_crystalizer.c:122
tests
static const struct @558 tests[]
checkasm_check_llauddsp
void checkasm_check_llauddsp(void)
Definition: llauddsp.c:108
AV_CPU_FLAG_AVX
#define AV_CPU_FLAG_AVX
AVX functions: requires OS support even if YMM registers aren't used.
Definition: cpu.h:51
csv
int csv
Definition: checkasm.c:470
AV_CPU_FLAG_FMA4
#define AV_CPU_FLAG_FMA4
Bulldozer FMA4 functions.
Definition: cpu.h:54
checkasm_check_vp9_loopfilter
void checkasm_check_vp9_loopfilter(void)
Definition: vp9dsp.c:452
cpu.h
AV_CPU_FLAG_AVX2
#define AV_CPU_FLAG_AVX2
AVX2 functions: requires OS support even if YMM registers aren't used.
Definition: cpu.h:56
i
#define i(width, name, range_min, range_max)
Definition: cbs_h264.c:63
checkasm_check_llviddsp
void checkasm_check_llviddsp(void)
Definition: llviddsp.c:183
checkasm_handle_signal
int checkasm_handle_signal(int s)
Definition: checkasm.c:1179
AV_CPU_FLAG_ARM_CRC
#define AV_CPU_FLAG_ARM_CRC
Definition: cpu.h:81
AV_CPU_FLAG_SSE2SLOW
#define AV_CPU_FLAG_SSE2SLOW
SSE2 supported, but usually not faster.
Definition: cpu.h:38
checkasm_check_vp9_mc
void checkasm_check_vp9_mc(void)
Definition: vp9dsp.c:559
AV_CPU_FLAG_NEON
#define AV_CPU_FLAG_NEON
Definition: cpu.h:73
size
int size
Definition: twinvq_data.h:10344
checkasm_check_vp3dsp
void checkasm_check_vp3dsp(void)
Definition: vp3dsp.c:113
checkasm_check_diracdsp
void checkasm_check_diracdsp(void)
Definition: diracdsp.c:85
DEF_CHECKASM_CHECK_FUNC
#define DEF_CHECKASM_CHECK_FUNC(type, fmt)
Definition: checkasm.c:1321
sysfd
int sysfd
Definition: checkasm.c:464
cmp
static av_always_inline int cmp(MPVEncContext *const s, const int x, const int y, const int subx, const int suby, const int size, const int h, int ref_index, int src_index, me_cmp_func cmp_func, me_cmp_func chroma_cmp_func, const int flags)
compares a block (either a full macroblock or a partition thereof) against a proposed motion-compensa...
Definition: motion_est.c:263
have_neon
#define have_neon(flags)
Definition: cpu.h:26
CheckasmFuncVersion::func
void * func
Definition: checkasm.c:437
checkasm_check_h264pred
void checkasm_check_h264pred(void)
Definition: h264pred.c:233
av_isdigit
static av_const int av_isdigit(int c)
Locale-independent conversion of ASCII isdigit.
Definition: avstring.h:202
AV_CPU_FLAG_FMA3
#define AV_CPU_FLAG_FMA3
Haswell FMA3 functions.
Definition: cpu.h:57
a
The reader does not expect b to be semantically here and if the code is changed by maybe adding a a division or other the signedness will almost certainly be mistaken To avoid this confusion a new type was SUINT is the C unsigned type but it holds a signed int to use the same example SUINT a
Definition: undefined.txt:41
ff_kperf_init
void ff_kperf_init(void)
Definition: macos_kperf.c:102
bench_runs
uint64_t bench_runs
Definition: checkasm.c:111
checkasm_check_huffyuvdsp
void checkasm_check_huffyuvdsp(void)
Definition: huffyuvdsp.c:95
line
Definition: graph2dot.c:48
float_near_abs_eps_array_ulp
int float_near_abs_eps_array_ulp(const float *a, const float *b, float eps, unsigned max_ulp, unsigned len)
Definition: checkasm.c:542
checkasm_get_perf_context
CheckasmPerf * checkasm_get_perf_context(void)
Definition: checkasm.c:1194
checkasm_check_fixed_dsp
void checkasm_check_fixed_dsp(void)
Definition: fixed_dsp.c:133
checkasm_check_colorspace
void checkasm_check_colorspace(void)
Definition: vf_colorspace.c:309
checkasm_check_utvideodsp
void checkasm_check_utvideodsp(void)
Definition: utvideodsp.c:90
is_negative
static int is_negative(union av_intfloat32 u)
Definition: checkasm.c:479
checkasm_check_audiodsp
void checkasm_check_audiodsp(void)
Definition: audiodsp.c:52
checkasm_check_huffyuvencdsp
void checkasm_check_huffyuvencdsp(void)
Definition: huffyuvencdsp.c:73
checkasm_check_sw_range_convert
void checkasm_check_sw_range_convert(void)
Definition: sw_range_convert.c:224
AV_CPU_FLAG_RV_MISALIGNED
#define AV_CPU_FLAG_RV_MISALIGNED
Fast misaligned accesses.
Definition: cpu.h:108
checkasm_check_mpegvideoencdsp
void checkasm_check_mpegvideoencdsp(void)
Definition: mpegvideoencdsp.c:191
AV_CPU_FLAG_SIMD128
#define AV_CPU_FLAG_SIMD128
Definition: cpu.h:112
checkasm_check_flacdsp
void checkasm_check_flacdsp(void)
Definition: flacdsp.c:155
checkasm_check_func
void * checkasm_check_func(void *func, const char *name,...)
Definition: checkasm.c:1100
AV_CPU_FLAG_SSE42
#define AV_CPU_FLAG_SSE42
Nehalem SSE4.2 functions.
Definition: cpu.h:48
checkasm_check_idctdsp
void checkasm_check_idctdsp(void)
Definition: idctdsp.c:95
CheckasmFuncVersion::next
struct CheckasmFuncVersion * next
Definition: checkasm.c:436
vsnprintf
#define vsnprintf
Definition: snprintf.h:36
common.h
use_color
static int use_color
Definition: log.c:128
AV_CPU_FLAG_RVV_I32
#define AV_CPU_FLAG_RVV_I32
Vectors of 8/16/32-bit int's *‍/.
Definition: cpu.h:99
checkasm_check_synth_filter
void checkasm_check_synth_filter(void)
Definition: synth_filter.c:47
cmp_func_names
static int cmp_func_names(const char *a, const char *b)
Definition: checkasm.c:729
AV_CPU_FLAG_SME_I16I64
#define AV_CPU_FLAG_SME_I16I64
Definition: cpu.h:83
AV_CPU_FLAG_ARMV8
#define AV_CPU_FLAG_ARMV8
Definition: cpu.h:74
checkasm_check_lpc
void checkasm_check_lpc(void)
Definition: lpc.c:96
len
int len
Definition: vorbis_enc_data.h:426
checkasm_check_g722dsp
void checkasm_check_g722dsp(void)
Definition: g722dsp.c:53
checkasm_check_ac3dsp
void checkasm_check_ac3dsp(void)
Definition: ac3dsp.c:193
PERF_STOP
#define PERF_STOP(t)
Definition: checkasm.h:421
is_red
#define is_red(f)
Definition: checkasm.c:754
AV_CPU_FLAG_ATOM
#define AV_CPU_FLAG_ATOM
Atom processor, some SSSE3 instructions are slower.
Definition: cpu.h:46
have_vfp
#define have_vfp(flags)
Definition: cpu.h:27
checkasm_check_svq1enc
void checkasm_check_svq1enc(void)
Definition: svq1enc.c:63
checkasm_check_vf_hflip
void checkasm_check_vf_hflip(void)
Definition: vf_hflip.c:72
ret
ret
Definition: filter_design.txt:187
checkasm_check_lls
void checkasm_check_lls(void)
Definition: lls.c:84
have_sme
#define have_sme(flags)
Definition: cpu.h:33
checkasm_check_scene_sad
void checkasm_check_scene_sad(void)
Definition: scene_sad.c:66
checkasm_check_blend
void checkasm_check_blend(void)
Definition: vf_blend.c:89
verbose
int verbose
Definition: checkasm.c:469
DEF_CHECKASM_CHECK_BODY
#define DEF_CHECKASM_CHECK_BODY(compare, type, fmt)
Definition: checkasm.c:1253
AV_CPU_FLAG_VFPV3
#define AV_CPU_FLAG_VFPV3
Definition: cpu.h:72
checkasm_check_bswapdsp
void checkasm_check_bswapdsp(void)
Definition: bswapdsp.c:60
checkasm_context_buf
checkasm_context checkasm_context_buf
Definition: checkasm.c:797
U
#define U(x)
Definition: vpx_arith.h:37
AV_CPU_FLAG_ARMV5TE
#define AV_CPU_FLAG_ARMV5TE
Definition: cpu.h:68
checkasm_check_vf_fspp
void checkasm_check_vf_fspp(void)
Definition: vf_fspp.c:165
AV_CPU_FLAG_MMX
#define AV_CPU_FLAG_MMX
standard MMX
Definition: cpu.h:32
suffix
const char * suffix
Definition: checkasm.c:369
checkasm_malloc
static void * checkasm_malloc(size_t size)
Definition: checkasm.c:634
AV_CPU_FLAG_RVI
#define AV_CPU_FLAG_RVI
I (full GPR bank)
Definition: cpu.h:94
checkasm_check_h264qpel
void checkasm_check_h264qpel(void)
Definition: h264qpel.c:51
random_seed.h
signal_handler
static void signal_handler(int s)
Definition: checkasm.c:843
checkasm_check_colordetect
void checkasm_check_colordetect(void)
Definition: vf_colordetect.c:135
AV_CPU_FLAG_AESNI
#define AV_CPU_FLAG_AESNI
Advanced Encryption Standard functions.
Definition: cpu.h:49
checkasm_bench_func
int checkasm_bench_func(void)
Definition: checkasm.c:1147
checkasm_check_mpegvideo_unquantize
void checkasm_check_mpegvideo_unquantize(void)
Definition: mpegvideo_unquantize.c:196
AV_CPU_FLAG_POWER8
#define AV_CPU_FLAG_POWER8
ISA 2.07.
Definition: cpu.h:66
cpu_flag_name
const char * cpu_flag_name
Definition: checkasm.c:467
AV_CPU_FLAG_SME
#define AV_CPU_FLAG_SME
Definition: cpu.h:80
checkasm_check_vf_sobel
void checkasm_check_vf_sobel(void)
Definition: vf_convolution.c:100
AV_CPU_FLAG_SSE
#define AV_CPU_FLAG_SSE
SSE functions.
Definition: cpu.h:36
ref
static int ref[MAX_W *MAX_W]
Definition: jpeg2000dwt.c:117
float_near_ulp_array
int float_near_ulp_array(const float *a, const float *b, unsigned max_ulp, unsigned len)
Definition: checkasm.c:502
checkasm_check_crc
void checkasm_check_crc(void)
Definition: crc.c:73
signal_handler_act
static const struct sigaction signal_handler_act
Definition: checkasm.c:838
AV_CPU_FLAG_MMXEXT
#define AV_CPU_FLAG_MMXEXT
SSE integer functions or AMD MMX ext.
Definition: cpu.h:33
Windows::Graphics::DirectX::Direct3D11::p
IDirect3DDxgiInterfaceAccess _COM_Outptr_ void ** p
Definition: vsrc_gfxcapture_winrt.hpp:53
checkasm_check_vp6dsp
void checkasm_check_vp6dsp(void)
Definition: vp6dsp.c:39
checkasm_check_av_tx
void checkasm_check_av_tx(void)
Definition: av_tx.c:102
checkasm_check_qpeldsp
void checkasm_check_qpeldsp(void)
Definition: qpeldsp.c:52
checkasm_check_vp9_itxfm
void checkasm_check_vp9_itxfm(void)
Definition: vp9dsp.c:311
checkasm_check_blackdetect
void checkasm_check_blackdetect(void)
Definition: vf_blackdetect.c:62
av_intfloat32::f
float f
Definition: intfloat.h:29
flag
int flag
Definition: checkasm.c:370
checkasm_check_v210enc
void checkasm_check_v210enc(void)
Definition: v210enc.c:83
checkasm_check_sbrdsp
void checkasm_check_sbrdsp(void)
Definition: sbrdsp.c:257
checkasm_check_hevc_deblock
void checkasm_check_hevc_deblock(void)
Definition: hevc_deblock.c:264
w
uint8_t w
Definition: llvidencdsp.c:39
test_pattern
const char * test_pattern
Definition: checkasm.c:468
usage
static int usage(const char *path)
Definition: checkasm.c:975
AV_CPU_FLAG_VFP_VM
#define AV_CPU_FLAG_VFP_VM
VFPv2 vector mode, deprecated in ARMv7-A and unavailable in various CPUs implementations.
Definition: cpu.h:75
AV_CPU_FLAG_MMI
#define AV_CPU_FLAG_MMI
Definition: cpu.h:86
checkasm_check_jpeg2000dsp
void checkasm_check_jpeg2000dsp(void)
Definition: jpeg2000dsp.c:92
checkasm_check_hevc_add_res
void checkasm_check_hevc_add_res(void)
Definition: hevc_add_res.c:87
int32_t
int32_t
Definition: audioconvert.c:56
float_near_ulp
int float_near_ulp(float a, float b, unsigned max_ulp)
Definition: checkasm.c:484
current_func_ver
CheckasmFuncVersion * current_func_ver
Definition: checkasm.c:455
check_cpu_flag
static void check_cpu_flag(const char *name, int flag)
Definition: checkasm.c:869
checkasm_context
sigjmp_buf checkasm_context
Definition: checkasm.h:75
bench_init_ffmpeg
static int bench_init_ffmpeg(void)
Definition: checkasm.c:931
checkasm_check_rv34dsp
void checkasm_check_rv34dsp(void)
Definition: rv34dsp.c:82
CheckasmFunc::versions
CheckasmFuncVersion versions
Definition: checkasm.c:446
isatty
#define isatty(fd)
Definition: checkasm.c:95
h
h
Definition: vp9dsp_template.c:2070
avg_cycles_per_call
static double avg_cycles_per_call(const CheckasmPerf *const p)
Definition: checkasm.c:683
AV_CPU_FLAG_RVV_I64
#define AV_CPU_FLAG_RVV_I64
Vectors of 64-bit int's *‍/.
Definition: cpu.h:101
check_err
static int check_err(const char *file, int line, const char *name, int w, int h, int *err)
Definition: checkasm.c:1240
cpu.h
CheckasmFunc::name
char name[1]
Definition: checkasm.c:448
checkasm_check_vvc_alf
void checkasm_check_vvc_alf(void)
Definition: vvc_alf.c:178
snprintf
#define snprintf
Definition: snprintf.h:34
av_log2
int av_log2(unsigned v)
Definition: intmath.c:26
checkasm_check_exrdsp
void checkasm_check_exrdsp(void)
Definition: exrdsp.c:77
balance_tree
static void balance_tree(CheckasmFunc **root)
Definition: checkasm.c:757
AV_CPU_FLAG_ARMV6T2
#define AV_CPU_FLAG_ARMV6T2
Definition: cpu.h:70
tsv
int tsv
Definition: checkasm.c:471
checkasm_check_llvidencdsp
void checkasm_check_llvidencdsp(void)
Definition: llvidencdsp.c:134
funcs
CheckasmFunc * funcs
Definition: checkasm.c:453
checkasm_check_opusdsp
void checkasm_check_opusdsp(void)
Definition: opusdsp.c:86